@@ -125,9 +125,12 @@ def fit_signatures(W, genome, metric="l2"):
     ### using NNLS algorithm 
     reg = nnls(W,genome)
     weights = reg[0]
+    est_genome = np.dot(W, weights)
     normalised_weights = weights/sum(weights)
     solution = normalised_weights*sum(genome)
     
+    
+    
     #print(W1)
     #convert the newExposure vector into list type structure
     newExposure = list(solution)
@@ -143,8 +146,7 @@ def fit_signatures(W, genome, metric="l2"):
     if np.sum(newExposure)!=maxmutation:
         newExposure[idxmaxcoef] = round(newExposure[idxmaxcoef])+maxmutation-sum(newExposure)
      
-    # compute the estimated genome
-    est_genome = np.dot(W, newExposure)
+    
     
     if metric=="cosine":
         newSimilarity = cos_sim(genome, est_genome)
@@ -179,6 +181,7 @@ def fit_signatures_pool(total_genome, W, index):
     ### using NNLS algorithm 
     reg = nnls(W,genome)
     weights = reg[0]
+    est_genome = np.dot(W, weights)
     normalised_weights = weights/sum(weights)
     solution = normalised_weights*sum(genome)
     
@@ -197,8 +200,7 @@ def fit_signatures_pool(total_genome, W, index):
     if np.sum(newExposure)!=maxmutation:
         newExposure[idxmaxcoef] = round(newExposure[idxmaxcoef])+maxmutation-sum(newExposure)
      
-    # compute the estimated genome
-    est_genome = np.dot(W, newExposure)
+    
     
     newSimilarity = cos_sim(genome, est_genome)
     
@@ -246,11 +248,12 @@ def add_signatures(W, genome, cutoff=0.05, presentSignatures=[], toBeAdded="all"
         if solver == "slsqp":
             sol = minimize(parameterized_objective2_custom, x0, args=(W1, genome),  bounds=bnds, constraints =cons1, tol=1e-30)
             newExposure = list(sol.x)
-        
+            est_genome = np.dot(W1, np.array(newExposure))
         if solver == "nnls":
         ### using NNLS algorithm        
             reg = nnls(W1, genome[:,0])
             weights = reg[0]
+            est_genome = np.dot(W1, weights)
             normalised_weights = weights/sum(weights)
             solution = normalised_weights*sum(np.sum(genome, axis=0))
             newExposure = list(solution)
@@ -270,8 +273,7 @@ def add_signatures(W, genome, cutoff=0.05, presentSignatures=[], toBeAdded="all"
         if np.sum(newExposure)!=maxmutation:
             newExposure[idxmaxcoef] = round(newExposure[idxmaxcoef])+maxmutation-sum(newExposure)
          
-        # compute the estimated genome
-        est_genome = np.dot(W1, newExposure)
+        
         if metric=="cosine":
             originalSimilarity = 1-cos_sim(genome[:,0], est_genome )
         elif metric=="l2":    
@@ -319,9 +321,12 @@ def add_signatures(W, genome, cutoff=0.05, presentSignatures=[], toBeAdded="all"
                 #the optimization step
                 sol = minimize(parameterized_objective2_custom, x0, args=(W1, genome),  bounds=bnds, constraints =cons1, tol=1e-30)
                 newExposure = list(sol.x)
+                # compute the estimated genome
+                est_genome = np.dot(W1, np.array(newExposure))
             if solver == "nnls":
                 reg = nnls(W1, genome[:,0])
                 weights = reg[0]
+                est_genome = np.dot(W1, weights)
                 normalised_weights = weights/sum(weights)
                 solution = normalised_weights*sum(genome)
                 newExposure = list(solution) #for nnls
@@ -340,8 +345,7 @@ def add_signatures(W, genome, cutoff=0.05, presentSignatures=[], toBeAdded="all"
             if np.sum(newExposure)!=maxmutation:
                 newExposure[idxmaxcoef] = round(newExposure[idxmaxcoef])+maxmutation-sum(newExposure)
              
-            # compute the estimated genome
-            est_genome = np.dot(W1, newExposure)
+            
             
             if metric=='cosine':
                 newSimilarity = 1-cos_sim(genome[:,0], est_genome)
@@ -406,18 +410,28 @@ def add_signatures(W, genome, cutoff=0.05, presentSignatures=[], toBeAdded="all"
 
 
 
-def remove_all_single_signatures(W, H, genomes, metric="ls", solver = "nnls", cutoff=0.05, background_sigs = [], verbose=False):
+def remove_all_single_signatures(W, H, genomes, metric="l2", solver = "nnls", cutoff=0.05, background_sigs = [], verbose=False):
     # make the empty list of the successfull combinations
+    signature_ids=sub.make_letter_ids(idlenth = W.shape[1], mtype = "Signature ")
+    
+    current_signatures=signature_ids
+    #print(current_signatures)
     successList = [0,[],0] 
     background_sig = copy.deepcopy(background_sigs)
+    
+    #setting the base_similarity
+    base_H_index = list(np.nonzero(H)[0])
+    reg = nnls(W[:,base_H_index],genomes)
+    base_weights = reg[0]
+    
     if metric == "cosine":
         # get the cos_similarity with sample for the oringinal W and H[:,i]
-        originalSimilarity= 1-cos_sim(genomes, np.dot(W, H))
+        originalSimilarity= 1-cos_sim(genomes, np.dot(W[:,base_H_index], base_weights))
         if verbose==True:
             print("originalSimilarity", 1-originalSimilarity)
     elif metric == "l2":
        
-        originalSimilarity = np.linalg.norm(genomes-np.dot(W, H) , ord=2)/np.linalg.norm(genomes, ord=2)
+        originalSimilarity = np.linalg.norm(genomes-np.dot(W[:,base_H_index], base_weights) , ord=2)/np.linalg.norm(genomes, ord=2)
         if verbose==True:
             print("originalSimilarity", originalSimilarity)
     # make the original exposures of specific sample round
@@ -433,6 +447,7 @@ def remove_all_single_signatures(W, H, genomes, metric="ls", solver = "nnls", cu
         else:
             return oldExposures, originalSimilarity, cos_sim(genomes, np.dot(W,H))    #first value is the exprosure, second value is the similarity (based on the similarity matic), third value is the cosine similarity
     # The while loop starts here
+    current_signatures=sub.get_items_from_index(signature_ids,np.nonzero(oldExposures)[0])
     while Flag: 
         
         # get the list of the indices those already have zero values
@@ -457,7 +472,7 @@ def remove_all_single_signatures(W, H, genomes, metric="ls", solver = "nnls", cu
         Winit = W[:,selectableIdx]
         
         # set the initial cos_similarity or other similarity distance
-        record  = [100, [], 1]   #
+        record  = [np.inf, [], 1]   #
         # get the number of current nonzeros
         l= Winit.shape[1]
         
@@ -470,7 +485,7 @@ def remove_all_single_signatures(W, H, genomes, metric="ls", solver = "nnls", cu
             
             loopSelection = list(range(l))
             del loopSelection[i]
-            #print (loopSelection)
+            
             W1 = Winit[:,loopSelection]
            
             
@@ -495,6 +510,7 @@ def remove_all_single_signatures(W, H, genomes, metric="ls", solver = "nnls", cu
                 ### using NNLS algorithm 
                 reg = nnls(W1,genomes)
                 weights = reg[0]
+                newSample = np.dot(W1, weights)
                 normalised_weights = weights/sum(weights)
                 solution = normalised_weights*sum(genomes)                
                 newExposure = list(solution)
@@ -504,24 +520,25 @@ def remove_all_single_signatures(W, H, genomes, metric="ls", solver = "nnls", cu
             
             #insert zeros in the required position the newExposure matrix
             initialZerosIdx.sort()
+            
             for zeros in initialZerosIdx:
                 newExposure.insert(zeros, 0)
             
             # get the maximum value the new Exposure
-            maxcoef = max(newExposure)
-            idxmaxcoef = newExposure.index(maxcoef)
+            #maxcoef = max(newExposure)
+            #idxmaxcoef = newExposure.index(maxcoef)
             
-            newExposure = np.round(newExposure)
+            #newExposure = np.round(newExposure)
             
             
-            if np.sum(newExposure)!=maxmutation:
-                newExposure[idxmaxcoef] = round(newExposure[idxmaxcoef])+maxmutation-sum(newExposure)
+            #if np.sum(newExposure)!=maxmutation:
+                #newExposure[idxmaxcoef] = round(newExposure[idxmaxcoef])+maxmutation-sum(newExposure)
                 
+            newExposure=np.array(newExposure)
             
-            newSample = np.dot(W, newExposure)
             if verbose==True:
-                print(newExposure)
-            
+                #print(newExposure)
+                print("\nRemoving {}".format(current_signatures[i]))
             if metric == "cosine":
                 newSimilarity = 1-cos_sim(genomes, newSample) 
                 if verbose==True:
@@ -530,8 +547,9 @@ def remove_all_single_signatures(W, H, genomes, metric="ls", solver = "nnls", cu
                 newSimilarity = np.linalg.norm(genomes-newSample, ord=2)/np.linalg.norm(genomes, ord=2)
                 if verbose==True:
                     print("newSimilarity",newSimilarity) 
-            difference =  newSimilarity -originalSimilarity 
-            
+            difference =  newSimilarity-originalSimilarity 
+            if difference<0:
+                difference=cutoff+1e-100
             if verbose==True:
                 print("difference", difference) 
             if difference<record[0]:
@@ -540,12 +558,19 @@ def remove_all_single_signatures(W, H, genomes, metric="ls", solver = "nnls", cu
                 #print("difference", difference)
             if verbose==True:
                 print("--------------------------------------")
+            
         if verbose==True:    
-            print("\n") 
-            print("Selected Exposure")
-            print(record[1])
+            print("\n############################\n############################") 
+            #print("Selected Exposure")
+            #print(record[1])
+            selected_sigs=sub.get_items_from_index(signature_ids,np.nonzero(record[1])[0])
+            dropped_sig=list(set(current_signatures)-set(selected_sigs))
+            print("Dropped Signature: {}".format(dropped_sig))
+            current_signatures=selected_sigs
             print("New Similarity", record[2])
-            print("****************************")
+            print("Similarity Difference: {}".format(record[2]-originalSimilarity))
+            print("Current Signatures: {}".format(selected_sigs))
+            print("****************************\n****************************\n\n")
             #print ("This loop's selection is {}".format(record))
         
         if record[0]>cutoff:   
@@ -556,6 +581,9 @@ def remove_all_single_signatures(W, H, genomes, metric="ls", solver = "nnls", cu
         else:
             successList = record
             background_sig = get_changed_background_sig_idx(list(record[1]), background_sig)
+           
+        originalSimilarity=record[2]
+        
         #print("The loop selection is {}".format(successList))
         
         #print (Flag)
@@ -724,11 +752,12 @@ def add_remove_signatures(W,
                           checkrule_penalty = 1.00, 
                           allsigids = False, 
                           directory = os.getcwd()+"/Signature_assaignment_logfile.txt", 
+                          connected_sigs=True,
                           verbose=False):
     
     lognote = open(directory, 'a')
  
-    
+    maxmutation=np.sum(np.array(sample))
     
     always_background = copy.deepcopy(permanent_sigs)
     M = sample   
@@ -765,10 +794,12 @@ def add_remove_signatures(W,
             loop_sig = [i] 
             
             background_sigs = list(set(background_sigs).union(set(always_background)))
-            background_sigs = add_connected_sigs(background_sigs, list(allsigids))
+            if connected_sigs==True:
+                background_sigs = add_connected_sigs(background_sigs, list(allsigids))
             
             add_exposures, add_distance, _ = add_signatures(W, M[:,np.newaxis], presentSignatures=copy.deepcopy(background_sigs), toBeAdded=loop_sig, 
-                                                  metric="l2", verbose=False, check_rule_negatives=check_rule_negatives, check_rule_penalty=1.00, cutoff=add_penalty) 
+                                                  metric="l2", verbose=False, check_rule_negatives=check_rule_negatives, check_rule_penalty=checkrule_penalty, cutoff=add_penalty) 
+            
             if verbose:
                 print("\n\n\n################## Add Index {} ########################".format(j)) 
                 print(np.nonzero(add_exposures)[0])
@@ -777,6 +808,7 @@ def add_remove_signatures(W,
                 
                 
             remove_exposures, remove_distance, _ = remove_all_single_signatures(W, add_exposures, M, background_sigs = always_background, metric="l2", verbose = False, cutoff=remove_penalty)
+            
             if verbose:
                 print("\n################## Remove ########################")
                 print(np.nonzero(remove_exposures)[0])
@@ -815,6 +847,17 @@ def add_remove_signatures(W,
             correlation,_=scipy.stats.pearsonr(M, N)
             correlation=round(correlation,2)
             break
+    
+    
+    # get the maximum value of the new Exposure
+    maxcoef = np.max(finalactivities)
+    idxmaxcoef = list(finalactivities).index(maxcoef)
+    
+    finalactivities = np.round(finalactivities)
+    
+    # We may need to tweak the maximum value of the new exposure to keep the total number of mutation equal to the original mutations in a genome
+    if np.sum(finalactivities)!=maxmutation:
+        finalactivities[idxmaxcoef] = round(finalactivities[idxmaxcoef])+maxmutation-np.sum(finalactivities) 
         
     if verbose:
         print("\n########################## Final ###########################")        
